/**
 * Copyright 2006-2007 by jFramework.org.
 */
package org.jframework.core.util;

import java.util.regex.Pattern;

/**
 * Regular expression factory.
 *
 * @author mgridnev
 * @version 1.0, Mar 10, 2005
 */
public abstract class Regex
{
    /**
     * Unlimited number of occurences
     */
    public static final int UNLIMITED = -1;

    /**
     * Completes sub expression
     */
    private static final Pattern complete(String sub)
    {
        return Pattern.compile("^" + sub + "$");
    }

    /**
     * OR-s two expressions together
     */
    private static String or(String one, String other)
    {
        return "((" + one + ")|(" + other + "))";
    }

    /**
     * Adds number of occurences constraint
     *
     * @param exp expression
     * @param min min number of occurences, a non negative integer
     * @param max max number of occurences, a positive integer or {@link #UNLIMITED}
     */
    private static String occurs(String exp, int min, int max)
    {
        if (min < 0) {
            throw new IllegalArgumentException("Min must be a non negative integer");
        }

        return "(" + exp + "){" + min + "," + (max < 0 ? "" : Integer.toString(max)) + "}";
    }

    /**
     * Regular expressions used for on-the-fly validation.  This type of regular expression is used to limit the type of
     * characters and total length of text the user is allowed to type.  It does not verify the validity of the final
     * input as it might prevent the user from modifying the text.  E.g. IP address validation should allow only '.' and
     * digits up to 15 characters long but not enforce the format of 0.0.0.0.
     */
    public abstract static class Relaxed
    {
        /**
         * Host name, i.e., letters, digits, and dash of up to 63 chars
         */
        private static final String HOST_NAME = occurs("[\\p{Alnum}\\-]", 0, 63);

        /**
         * IP Address, i.e., digits and dot of up to 15 chars
         */
        private static final String IP_ADDRESS = occurs("[\\d\\.]", 0, 15);

        /**
         * Domain name, i.e., letters, digits, dash, and period of up to 128 chars
         */
        private static final String DOMAIN = occurs("[\\p{Alnum}\\-\\.]", 0, 128);

        /**
         * Returns a pattern matching any character of up to <code>max</code>
         *
         * @param max maximum length larger than 1
         * @return pattern
         */
        public static Pattern text(int max)
        {
            if (max < 1) {
                throw new IllegalArgumentException("Length must be positive integer: " + max);
            }

            return Pattern.compile("^(.){0," + max + "}$", Pattern.DOTALL);
        }

        /**
         * Returns a pattern matching only digits of up to <code>max</code>
         *
         * @param max maximum length specify <code>-1</code> or {@link Regex#UNLIMITED} to not limit the length
         * @return pattern
         */
        public static Pattern digits(int max)
        {
            return complete(occurs("\\d", 0, max));
        }

        /**
         * Returns a pattern matching only hexadecimal digits of up to <code>max</code>
         *
         * @param max maximum length specify <code><= 0</code> to not limit the length
         * @return pattern
         */
        public static Pattern hex(int max)
        {
            return complete(occurs("\\p{XDigit}", 0, max));
        }

        /**
         * Returns a pattern matching an IP address
         */
        public static Pattern IP()
        {
            return complete(IP_ADDRESS);
        }

        /**
         * Returns a pattern matching an IP address
         */
        public static Pattern netMask()
        {
            return IP();
        }

        /**
         * Returns a pattern matching host name or IP address i.e. any cobination of letters, nubmers, and dash of up to
         * 63 characters
         *
         * @return pattern
         */
        public static Pattern hostName()
        {
            return complete(HOST_NAME);
        }

        /**
         * Returns a pattern matching a valid domain name or ip address.  Similar to {@link #host()} only 128 long
         */
        public static Pattern domain()
        {
            return Pattern.compile(DOMAIN);
        }


        /**
         * Returns a pattern matching host name or IP address i.e. any cobination of letters, nubmers, period, and dash
         * of up to 63 characters
         *
         * @return pattern
         */
        public static Pattern host()
        {
            return complete(or(or(IP_ADDRESS, HOST_NAME), DOMAIN));
        }

        /**
         * Returns a pattern matching a port name i.e. up to 5 digits
         */
        public static Pattern port()
        {
            return digits(5);
        }

        /**
         * Returns a pattern matching MD5 check sum
         */
        public static Pattern MD5()
        {
            return complete(occurs("\\p{XDigit}", 0, 32));
        }
    }

    /**
     * Regular expressions used for input validation.  This type of regular expression is used to verify the final
     * input.  E.g. IP address validation should eforce the exact octet syntax of '000.000.000.000'.
     */
    public abstract static class Strict
    {
        /**
         * IP address
         */
        private static final String IP = "(\\d{1,2}|1\\d\\d|2[0-4]\\d|25[0-5])\\.(\\d{1,2}|1\\d\\d|2[0-4]\\d|25[0-5])" +
            "\\.(\\d{1,2}|1\\d\\d|2[0-4]\\d|25[0-5])\\.(\\d{1,2}|1\\d\\d|2[0-4]\\d|25[0-5])";

        /**
         * Host name
         */
        private static final String HOST_NAME = "\\p{Alpha}([\\p{Alnum}\\\\-]{0,61}\\p{Alnum})?";

        /**
         * Domain name
         */
        private static final String DOMAIN = "(\\p{Alnum}([\\p{Alnum}\\-]{0,61}\\p{Alnum})?\\.)+\\p{Alpha}{2,6}";

        /**
         * A host, which may be an IP address, host, or domain
         */
        private static final String HOST = or(or(DOMAIN, IP), HOST_NAME);

        /**
         * Port number (approx)
         */
        private static final String PORT = "(\\d){1,5}";

        /**
         * URL based on RFC 1738
         */
        private static interface URL
        {
            /**
             * Uchar as defined by RFC 1738
             */
            String UCHAR = "(?:[\\w$\\-_\\.\\+\\!\\*'\\(\\),]?(?:%\\p{XDigit}{2})?)";

            /**
             * URI segment as defined by RFC 1738
             */
            String HSEGMENT = occurs(or(UCHAR, "[;:&@= ]?"), 0, UNLIMITED); // RFC does not allow spaces, we do

            /**
             * Search as defined by RFC 1738
             */
            String SEARCH = HSEGMENT;

            /**
             * User as defined RFC 1738
             */
            String USER = occurs(or(UCHAR, "[;?&=]?"), 1, UNLIMITED);

            /**
             * Password as defined RFC 1738
             */
            String PASSWORD = USER;

            /**
             * Hpath as defined by RFC 1738
             */
            String HPATH = HSEGMENT + occurs("/" + HSEGMENT, 0, UNLIMITED);
        }

        /**
         * At least one non white space character
         */
        public static Pattern notEmpty()
        {
            return complete(".*\\p{Graph}.*");
        }

        /**
         * A line of text at least <tt>min</tt> characters long but no exceeding <tt>max</tt>.
         *
         * @param min min text length, inclusive, specify zero if min lenght is not required
         * @param max max text length, inclusive
         * @return a regex pattern
         */
        public static Pattern text(int min, int max)
        {
            return Pattern.compile("^(.){" + min + "," + max + "}$", Pattern.DOTALL);
        }

        /**
         * Returns a pattern matching only digits of atleast <code>min</code> and up to <code>max</code>
         *
         * @param min minimum length
         * @param max maximum length
         * @return pattern
         */
        public static Pattern digits(int min, int max)
        {
            return complete(occurs("\\d", min, max));
        }

        /**
         * Returns a pattern matching only hex digits of atleast <code>min</code> and up to <code>max</code>
         *
         * @param min minimum length
         * @param max maximum length
         * @return pattern
         */
        public static Pattern hex(int min, int max)
        {
            return complete(occurs("\\p{XDigit}", min, max));
        }

        /**
         * Returns a pattern matching an IP address
         */
        public static Pattern IP()
        {
            return complete(IP);
        }

        /**
         * Returns a pattern matching a net mask
         */
        public static Pattern netMask()
        {
            return complete("(0|128|192|224|240|248|252|254)(\\.0){3}|" +
                "255\\.(0|128|192|224|240|248|252|254)(\\.0){2}|" +
                "255\\.255\\.(0|128|192|224|240|248|252|254)\\.0|" +
                "255(\\.255){2}\\.(0|128|192|224|240|248|252|254|255)");
        }

        /**
         * Returns a pattern matching host name or IP address i.e. any cobination of letters, nubmers, period, and dash
         * from 1 to 63 characters
         *
         * @return pattern
         */
        public static Pattern hostName()
        {
            return complete(HOST_NAME);
        }

        /**
         * Returns a pattern matching a valid domain name only e.g. www.domain.com.  Borrowed from <a
         * href="http://regexlib.com/REDetails.aspx?regexp_id=391">http://regexlib.com/REDetails.aspx?regexp_id=391</a>.
         */
        public static Pattern domain()
        {
            return complete(DOMAIN);
        }

        /**
         * Returns a pattern matching a host, which may be an IP address, a host name, or a domain name
         *
         * @return pattern
         */
        public static Pattern host()
        {
            return complete(HOST);
        }

        /**
         * Returns a pattern matching a port number (approx), i.e., up to 5 digits
         */
        public static Pattern port()
        {
            return complete(PORT);
        }

        /**
         * Returns a pattern matching MD5 check sum
         */
        public static Pattern MD5()
        {
            return hex(32, 32);
        }

        /**
         * Returns a pattern matching a URL
         *
         * @param protocols an array of protocol e.g. "http" or "https" of size 1 or larger.
         * @return regular expression
         */
        public static Pattern URL(String[] protocols)
        {
            String protocol = protocols[0];
            for (int i = 1; i < protocols.length; i++) {
                protocol = or(protocol, protocols[i]);
            }

            return complete(protocol +
                "://" + occurs(URL.USER + occurs(":" + URL.PASSWORD, 0, 1) + "@", 0, 1) +
                HOST + occurs(":" + PORT, 0, 1) + occurs("/" + URL.HPATH, 0, UNLIMITED) +
                occurs("\\?" + URL.SEARCH, 0, UNLIMITED));
        }
    }
}
