/**
 * Copyright 2006-2008 by jFramework.org.
 */
package org.jframework.core.util;

import java.text.MessageFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

/**
 * <p>Resource mapper built on top of {@link ResourceBundle} to transparently resolve externalized strings. It is more convenient to use than the traditional resource bundle.</p>
 * <p>E.g., <tt>Resource.getString("org.framework.res:key")</tt> looks up a string in <tt>org/jframework/res</tt> properties file with <tt>key</tt> key.</p>
 *
 * @author mgridnev
 * @version 2.0, Jun 1, 2008
 */
public class Resources implements LocaleAware
{
    /**
     * Logger
     */
    private static final Logger log = Logger.getLogger(Resources.class);

    /**
     * Resource name pattern
     */
    private static final Pattern pattern = Pattern.compile("^((?:\\p{Alpha}[\\w$]*\\.)*\\p{Alpha}[\\w$]*)\\:(.+)$");

    /**
     * Loaded resource bundles
     */
    private final Map<String, ResourceBundle> bundles = new HashMap<String, ResourceBundle>();

    /**
     * Locale associated with the resource.
     */
    private Locale locale;

    /**
     * Creates a resource associated with the default locale.
     */
    public Resources()
    {
        this(Locale.getDefault());
    }

    /**
     * Creates a resource associated with a particular locale. All resource lookups will be relative to the locale.
     *
     * @param locale locale
     */
    public Resources(final Locale locale)
    {
        this.locale = locale;
    }

    /**
     * Returns a locale associated with this resource.
     *
     * @return local
     */
    public Locale getLocale()
    {
        return locale;
    }

    /**
     * Sets a new local associated with this resource.
     *
     * @param locale locale
     */
    public void setLocale(Locale locale)
    {
        if (!this.locale.equals(locale)) {
            this.locale = locale;
            bundles.clear(); // clear cached bundles
        }
    }

    /**
     * Returns a map of cached resource bundles keyed off context names
     *
     * @return resource bundles
     */
    public Map<String, ResourceBundle> getBundles()
    {
        return bundles;
    }

    /**
     * Loads a resource bundle or returns a previously loaded bundle.
     *
     * @param name resource bundle name in the dotted notation
     * @return loaded resource bundle or <tt>null</tt> if a bundle cannot be loaded
     */
    public ResourceBundle lookUpBundle(final String name)
    {
        // Try to get an existing bundle. Note, because we cache nulls we have to check for key existence explicitly
        if (bundles.containsKey(name)) {
            return bundles.get(name);
        }

        // Try to load
        ResourceBundle bundle = null;
        try {
            bundle = ResourceBundle.getBundle(name, locale);
        } catch (MissingResourceException e) {
            log.warn("Resource bundle " + name + " not found");
        }

        // Cache either the loaded bundle or null. Yes, we cache nulls to speed up future lookups
        bundles.put(name, bundle);

        return bundle;
    }

    /**
     * Returns a string value.
     *
     * @param resource key in the form <tt>"context:key"</tt> where <tt>context</tt> is a resource bundle name and <tt>key</tt> a key.
     * @return looked up string value if it can be resolved, <tt>resource</tt> otherwise
     */
    public String getString(final String resource)
    {
        final Matcher matcher = pattern.matcher(resource);
        if (!matcher.matches()) {
            return resource;
        }

        return getString(matcher.group(1), matcher.group(2), resource);
    }

    /**
     * Looks up a string and uses it with {@link java.text.MessageFormat} to produce the final output.
     *
     * @param resource key in the form <tt>"context:key"</tt> where <tt>context</tt> is a resource bundle name and <tt>key</tt> a key.
     * @param args     format arguments
     * @return formatted string
     */
    public String formatString(String resource, Object... args)
    {
        return new MessageFormat(getString(resource)).format(args);
    }

    /**
     * Returns a string value.
     *
     * @param context      resource bundle name
     * @param key          key in the resource bundle
     * @param defaultValue value to return if the lookup fails
     * @return mapped string or a string in the form <tt>"context:key"</tt> if either key or context cannot be found
     */
    public String getString(final String context, final String key, final String defaultValue)
    {
        final ResourceBundle bundle = lookUpBundle(context);

        if (bundle != null) {
            try {
                return bundle.getString(key);
            } catch (MissingResourceException e) {
                log.warn("Resource " + key + " not found in " + context + " bundle.");
            }
        }

        return defaultValue;
    }
}
