/**
 * Copyright 2006-2007 by jFramework.org.
 */
package org.jframework.core.util;

import java.io.StringWriter;
import java.util.*;

import javax.swing.*;

import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * String related utilities.
 *
 * @author mgridnev
 * @version 2.0, Feb 21, 2007
 */
public class StringUtils
{
    /**
     * String representation of <code>null</code>.
     */
    public static final String NULL = "<null>";

    /**
     * Default object to string converter
     */
    public static final ToStringConverter DEFAULT_TO_STRING = new ToStringConverter()
    {
        public String toString(Object object)
        {
            return object == null ? NULL : object.toString();
        }
    };

    /**
     * Reflection object to string converter
     */
    public static final ToStringConverter REFLECTION_TO_STRING = new ToStringConverter()
    {
        public String toString(Object object)
        {
            return ToStringBuilder.reflectionToString(object);
        }
    };

    /**
     * Utility
     */
    private StringUtils()
    {
    }

    /**
     * Truncates a line not to exceed a maximum of <tt>start + end</tt> characters inserting an ellipsis if necessary. Here are possible combinations: <table cellpadding="5"
     * cellspacing="0" border="1"> <tr><th>Input</th><th>Limit</th><th>Output</th></tr> <tr><td>The quick brown fox</td><td>5, 5</td><td>The q...n fox</td></tr> <tr><td>The quick
     * brown fox</td><td>5, 0</td><td>The q...</td></tr> <tr><td>The quick brown fox</td><td>0, 5</td><td>...n fox</td></tr> </table>
     *
     * @param start maximim prefix size
     * @param end   maximim suffix size
     * @return truncated string
     */
    public static String truncate(final String input, final int start, final int end)
    {
        final StringBuffer sb = new StringBuffer(input);

        final String prefix = sb.length() >= start ? sb.substring(0, start) : sb.toString();
        sb.delete(0, prefix.length());

        final String suffix = sb.length() >= end ? sb.substring(sb.length() - end) : sb.toString();

        return prefix.length() + suffix.length() < input.length() ? prefix + "..." + suffix : input;
    }

    /**
     * Ensures input string is at least the given length.  If the string length is equal to or greater than the given lengh, no changes are made.
     *
     * @param input  input string
     * @param ch     padding char
     * @param length minimum required length
     * @return padded string
     */
    public static String pad(String input, final char ch, final int length)
    {
        final int diff = length - input.length();

        if (diff > 0) {
            final char[] rgPadding = new char[diff];
            Arrays.fill(rgPadding, ch);

            input += new String(rgPadding);
        }

        return input;
    }

    /**
     * Replaces all occurences of <tt>what</tt> in the <tt>string</tt> even if <tt>what</tt> contains special chars.
     */
    public static String replaceAll(final String string, final String what, final String replacement)
    {
        final StringBuffer sb = new StringBuffer(string);

        for (int index = 0; ;) {
            index = sb.indexOf(what, index);

            if (index < 0) {
                break;
            }

            sb.replace(index, index + what.length(), replacement);
            index += replacement.length() + 2;
        }

        return sb.toString();
    }

    /**
     * Escapes all characters in the string
     *
     * @param string   string to escape
     * @param toEscape all illegal characters that need to be escaped
     * @param chEscape escape character
     * @return escaped string
     */
    public static String escape(final String string, final String toEscape, final char chEscape)
    {
        final StringBuffer sb = new StringBuffer(string);

        for (int i = 0; i < sb.length(); i ++) {
            if (toEscape.indexOf(sb.charAt(i)) != -1) {
                sb.insert(i ++, chEscape);
            }
        }

        return sb.toString();
    }

    /**
     * Returns a string of the given length filled with the given char
     */
    public static String toString(final char ch, final int length)
    {
        final char[] rg = new char[length];
        Arrays.fill(rg, ch);

        return new String(rg);
    }

    /**
     * Returns the object string representation
     *
     * @param object    object
     * @param converter to string converter, may be <tt>null</tt>
     * @return object to string representation
     */
    public static String toString(Object object, ToStringConverter converter)
    {
        return (converter == null ? DEFAULT_TO_STRING : converter).toString(object);
    }

    /**
     * Returns a list of elements as a string.
     *
     * @param col       a list of elements
     * @param separator separator string to insert between elements
     * @param fEscape   whether the occurences of separator string inside list elements should be escaped e.g. if separator is "," and list item is "item, 1", the resulting string
     *                  is "item\\, 1"
     * @param converter an optional object to string converter
     * @return their combined string representation
     */
    public static String toString(final Collection col, final String separator, final boolean fEscape, ToStringConverter converter)
    {
        final StringBuffer sb = new StringBuffer();
        for (Object element : col) {
            String string = toString(element, converter);

            if (fEscape) {
                string = replaceAll(string, separator, "\\" + separator);
            }

            sb.append(string);
            sb.append(separator);
        }

        if (sb.length() > 0) {
            final int cc = sb.length();
            sb.delete(cc - separator.length(), cc); // remove last occurence of separator
        }

        return sb.toString();
    }

    /**
     * Converts HTML to plain text
     */
    public static String toPlainText(String string)
    {
        // Replace <br> with <p/> because <br> are not property processed
        string = string.replaceAll("<br>", "<p>"); // strangely but the parser does not properly process XHTML

        // Load as HTML
        final JEditorPane pane = new JEditorPane("text/html", string);

        // Write out as plain text
        final StringWriter out = new StringWriter();

        try {
            pane.getEditorKitForContentType("text/plain").write(out, pane.getDocument(), 0,
                pane.getDocument().getLength());
        } catch (Exception e) {
            return string; // should never happen
        }

        return out.toString().trim();  // remove extra LF
    }

    /**
     * Normalizes HTML by getting rid of embedded &lt;html&gt; and &lt;body&gt; tags
     */
    public static String normalizeHTML(String string)
    {
        string = replaceAll(string, "<body>", "");
        string = replaceAll(string, "</body>", "");
        string = replaceAll(string, "<html>", "");
        string = replaceAll(string, "</html>", "");

        return "<html><body>" + string + "</body></html>";
    }

    /**
     * Converts plain text to HTML
     */
    public static String toHTML(String string)
    {
        // Load as HTML
        string = replaceAll(string, "&", "&amp;");
        string = replaceAll(string, "\"", "&quot;");
        string = replaceAll(string, "<", "&lt;");
        string = replaceAll(string, ">", "&gt;");

        string = string.replaceAll("\n", "<br>");

        return "<html><body>" + string + "</body></html>";
    }

    /**
     * Prints a byte array as a string of hex digits
     *
     * @param rgBytes   byte array
     * @param separator byte separator, may be <tt>'\0'</tt>
     * @return hex string
     */
    public static String toHexString(final byte[] rgBytes, final char separator)
    {
        final StringBuffer sb = new StringBuffer();

        for (int i = 0; i < rgBytes.length; i++) {
            final byte b = rgBytes[i];
            sb.append(Integer.toHexString(b >> 4 & 0x0F));
            sb.append(Integer.toHexString(b & 0x0F));

            if (i < rgBytes.length - 1 && separator != '\0') {
                sb.append(separator);
            }
        }

        return sb.toString().toUpperCase();
    }

    /**
     * Converts a comma-separated list of values into a list
     *
     * @param string comma-separated list
     * @return a list of values
     */
    public static List<String> toList(final String string)
    {
        if (string.length() == 0) {
            return Collections.emptyList();
        }

        final String[] rgParts = string.split("(?<=[^\\\\]),");

        final List<String> list = new LinkedList<String>();
        for (String rgPart : rgParts) {
            final String part = rgPart.trim().replaceAll("\\\\,", "\\,");

            list.add(NULL.equals(part) ? null : part);
        }

        return list;
    }

    /**
     * Check for String being <code>null</code> or an empty string
     *
     * @param str string to test
     * @return <code>true</code> if the input string is "empty", <code>false</code> otherwise
     */
    public static boolean isEmpty(final String str)
    {
        return str == null || str.length() == 0;
    }

    /**
     * Object to string converter.
     */
    public interface ToStringConverter
    {
        /**
         * Returns string representation of the object
         */
        String toString(Object object);
    }
}
