/**
 * Copyright 2008 by jFramework.org.
 */
package org.jframework.core.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.TimeZone;

import org.apache.commons.lang.ObjectUtils;

/**
 * An implementation of {@link TimeZoneAware} that servers as a container for other time zone aware objects. It propgates the changes to the time zone to all of its children.
 *
 * @author mgridnev
 * @version 1.0, Jun 10, 2008
 */
public class TimeZoneAwareContainer implements TimeZoneAware
{
    /**
     * Current time zone.
     */
    private TimeZone timeZone = TimeZone.getDefault();

    /**
     * Contained children.
     */
    private final List<TimeZoneAware> children = new ArrayList<TimeZoneAware>();

    /**
     * {@inheritDoc}
     */
    public final TimeZone getTimeZone()
    {
        return timeZone;
    }

    /**
     * {@inheritDoc}
     */
    public final void setTimeZone(TimeZone timeZone)
    {
        if (ObjectUtils.equals(this.timeZone, timeZone)) {
            return; // already up to date
        }

        this.timeZone = timeZone;

        for (TimeZoneAware child : children) {
            child.setTimeZone(timeZone);
        }
    }

    /**
     * Returns container children.
     *
     * @return an immutable list
     */
    public List<TimeZoneAware> getChildren()
    {
        return Collections.unmodifiableList(children);
    }

    /**
     * Adds a child and synchronizes child's time zone with the container's
     *
     * @param child time zone aware child
     */
    public final void addChild(TimeZoneAware child)
    {
        if (child == null) {
            throw new NullPointerException("Child may not be null.");
        }

        child.setTimeZone(timeZone);

        children.add(child);
    }

    /**
     * Removes a child.
     *
     * @param child child to remove
     */
    public final void removeChild(TimeZoneAware child)
    {
        children.remove(child);
    }
}
